﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Petzold.Media3D;

namespace CircularGradientBrushDemo
{
    public partial class CircularGradientBrushControl : UserControl
    {
        public static readonly DependencyProperty Color1Property =
            DependencyProperty.Register("Color1",
                typeof(Color),
                typeof(CircularGradientBrushControl),
                new PropertyMetadata(Colors.Blue, OnColorChanged));

        public static readonly DependencyProperty Color2Property =
            DependencyProperty.Register("Color2",
                typeof(Color),
                typeof(CircularGradientBrushControl),
                new PropertyMetadata(Colors.Red, OnColorChanged));

        public static readonly DependencyProperty ThicknessProperty =
            DependencyProperty.Register("Thickness",
                typeof(double),
                typeof(CircularGradientBrushControl),
                new PropertyMetadata(48.0, OnThicknessChanged));

        public static readonly DependencyProperty AngleProperty =
            DependencyProperty.Register("Angle",
                typeof(double),
                typeof(CircularGradientBrushControl),
                new PropertyMetadata(OnAngleChanged));

        TaperTransform taperTransformTop = new TaperTransform();
        TaperTransform taperTransformRight = new TaperTransform();
        TaperTransform taperTransformBottom = new TaperTransform();
        TaperTransform taperTransformLeft = new TaperTransform();

        public CircularGradientBrushControl()
        {
            InitializeComponent();

            SizeChanged += new SizeChangedEventHandler(OnSizeChanged);
        }

        public Color Color1
        {
            set { SetValue(Color1Property, value); }
            get { return (Color)GetValue(Color1Property); }
        }

        public Color Color2
        {
            set { SetValue(Color2Property, value); }
            get { return (Color)GetValue(Color2Property); }
        }

        public double Thickness
        {
            set { SetValue(ThicknessProperty, value); }
            get { return (double)GetValue(ThicknessProperty); }
        }

        public double Angle
        {
            set { SetValue(AngleProperty, value); }
            get { return (double)GetValue(AngleProperty); }
        }

        static void OnColorChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as CircularGradientBrushControl).OnColorChanged(args);
        }

        void OnColorChanged(DependencyPropertyChangedEventArgs args)
        {
            brushTop.GradientStops[0].Color = Color1;
            brushRight.GradientStops[0].Color = Color1;
            brushBottom.GradientStops[0].Color = Color1;
            brushLeft.GradientStops[0].Color = Color1;

            brushTop.GradientStops[1].Color = Color2;
            brushRight.GradientStops[1].Color = Color2;
            brushBottom.GradientStops[1].Color = Color2;
            brushLeft.GradientStops[1].Color = Color2;
        }

        static void OnThicknessChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as CircularGradientBrushControl).OnThicknessChanged(args);
        }

        void OnThicknessChanged(DependencyPropertyChangedEventArgs args)
        {
            SetInnerGeometry(new Size(ActualWidth, ActualHeight));
        }

        static void OnAngleChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as CircularGradientBrushControl).rotateTransform.Angle = 135 + (double)args.NewValue;
        }

        void OnSizeChanged(object sender, SizeChangedEventArgs args)
        {
            taperTransformTop.TaperSide = TaperSide.Bottom;
            taperTransformTop.TargetHeight = args.NewSize.Height / 2;
            taperTransformTop.TargetWidth = args.NewSize.Width;
            taperTransformTop.TaperCorner = TaperCorner.Both;
            taperTransformTop.TaperFraction = 0.01;
            projectionTop.ProjectionMatrix = taperTransformTop.Matrix3D;

            taperTransformRight.TaperSide = TaperSide.Left;
            taperTransformRight.TargetHeight = args.NewSize.Height;
            taperTransformRight.TargetWidth = args.NewSize.Width / 2;
            taperTransformRight.TaperCorner = TaperCorner.Both;
            taperTransformRight.TaperFraction = 0.01;
            projectionRight.ProjectionMatrix = taperTransformRight.Matrix3D;

            taperTransformBottom.TaperSide = TaperSide.Top;
            taperTransformBottom.TargetHeight = args.NewSize.Height / 2;
            taperTransformBottom.TargetWidth = args.NewSize.Width;
            taperTransformBottom.TaperCorner = TaperCorner.Both;
            taperTransformBottom.TaperFraction = 0.01;
            projectionBottom.ProjectionMatrix = taperTransformBottom.Matrix3D;

            taperTransformLeft.TaperSide = TaperSide.Right;
            taperTransformLeft.TargetHeight = args.NewSize.Height;
            taperTransformLeft.TargetWidth = args.NewSize.Width / 2;
            taperTransformLeft.TaperCorner = TaperCorner.Both;
            taperTransformLeft.TaperFraction = 0.01;
            projectionLeft.ProjectionMatrix = taperTransformLeft.Matrix3D;

            ellipseGeoOuter.Center = new Point(args.NewSize.Width / 2, args.NewSize.Height / 2);
            ellipseGeoOuter.RadiusX = args.NewSize.Width / 2;
            ellipseGeoOuter.RadiusY = args.NewSize.Height / 2;

            ellipseGeoInner.Center = new Point(args.NewSize.Width / 2, args.NewSize.Height / 2);
            SetInnerGeometry(args.NewSize);
        }

        void SetInnerGeometry(Size size)
        {
            ellipseGeoInner.RadiusX = size.Width / 2 - Thickness;
            ellipseGeoInner.RadiusY = size.Height / 2 - Thickness;
        }
    }
}
